@-moz-document url-prefix('about:preferences') {
  /* Hide the home category */
  /* Not needed because it only configures the home and new tab page, which are not accessible */
  #category-home {
    display: none;
  }

  /* Hide the default browser box */
  /* Not needed because PWAsForFirefox browser installation is not supposed to be made default */
  #defaultBrowserBox,
  setting-control:has(> #alwaysCheckDefault),
  setting-control:has(> #isDefaultPane),
  setting-control:has(> #isNotDefaultPane) {
    display: none;
  }

  /* Hide the layout (vertical tabs) group */
  /* Hidden until we figure out how to properly support them (#667) */
  groupbox[data-subcategory="layout"] {
    display: none;
  }

  /* Hide native window controls preference when CSD is not available */
  /* Because it doesn't have effect on those platforms */
  @media not (-moz-gtk-csd-available) {
    .pref-csd-only {
      display: none;
    }
  }

  /* Remove margin on global input elements */
  .global-input {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }
}
