// Any comment. You must start the file with a single-line comment!

pref('general.config.filename', '_autoconfig.cfg');
pref('general.config.obscure_value', 0);
pref('general.config.sandbox_enabled', false);
